/*
  Author:  Jonathan Bondzie, Dr. Chuck Yeung
           Lennard Gas Simulation Source

	   Copyright (C) 1999  Penn State University

	   This program is free software; you can redistribute it and/or
	   modify it under the terms of the GNU General Public License
	   as published by the Free Software Foundation;
	   
	   This program is distributed in the hope that it will be useful,
	   but WITHOUT ANY WARRANTY; without even the implied warranty of
	   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	   GNU General Public License for more details.
	   
	   You should have received a copy of the GNU General Public License
	   along with this program; if not, write to the Free Software
	   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
	   02111-1307, USA.

	   email jab341@psu.edu
*/

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <iomanip.h>

#include	"kiss.h"

/* header files for the graphics programs -- must be included for graphics */

#include        "motifR.h"
#include        "Headers.h" 

/* constant parameters and macros */
#define SQUARE(x)	( (x)*(x) )

#define R_max 2      /* pretty cool name for a variable huh ? R_max is the constant value 
			where all the forces are set to zero due significance  compared to 
			the energy value.    */


#define R_maxSq (R_max*R_max)          //R_max squared
#define R_check  (R_max * 1.5)         /* R_check is the neighbour list radius*/
#define vconstant 299.3793626
#define tempconstant 862.507157 /* conversion factor for the temperature */
#define energyconstant 1.19E-20 /* conversion factor for the energies */

/*********************** functions ********************************/

void VWall_Pressure(Variables* ptr); // prototype
Boolean CommandLineParser(int argc, char** argv,Variables* ptr,Boolean nographics);//prototype

/* this routine will calculate the average velocity at time dt
   then create a histogram for the maxwell boltzman distribution
   based on the Vrms values.                                     */

int bhv =0;                   /* the biggest histogram value */
void MaxBoltzMemAlloc(float avg_velocity,Variables* ptr)
{
  /* the speed that will likely be the max, where
     most particles will be at                    */
  
  static float probable_speed = avg_velocity;

  (*ptr).binsize = probable_speed/30;

  bhv = (int)(probable_speed*300) ;/*this will give us 300 times max speed/ 30 bins */

  /* we are going to store the speeds of the particles based on thier
     bin positions. So to determine the biggest array needed to accomodate 
     this, we will assume the biggest velocity will be bo greater
     than avg_velocity*5*.82(that's five times the probable speed.
     If this is the case then to determine the bin position of 
     at some speed we will divide that speed v/(probable_speed*5). this should
     hopefully accomodate all speeds.                                          */

 
  (*ptr).histogram = new unsigned int[bhv];
  
  for(int i=0;i<bhv;i++)
    (*ptr).histogram[i]= 0;
}

void InitializeArrays(Variables* ptr)
{
  static int first_time = TRUE;
  
  if(first_time == FALSE )
  {
     delete[] (*ptr).x;
     delete[] (*ptr).y;
     delete[] (*ptr).vx;
     delete[] (*ptr).vy;
     delete[] (*ptr).fx;
     delete[] (*ptr).fy;
     delete[] (*ptr).fy0;
     delete[] (*ptr).fx0;
     delete[] (*ptr).prev_x;
     delete[] (*ptr).prev_y;
     delete[] (*ptr).neighbours;
  }
  
    
 /* create all the arrays with thier their proper dimensions */

  (*ptr).x      = new double[(*ptr).nparticles];
  (*ptr).y      = new double[(*ptr).nparticles];
  (*ptr).vx     = new double[(*ptr).nparticles];
  (*ptr).vy     = new double[(*ptr).nparticles];
  (*ptr).fx     = new float[(*ptr).nparticles];
  (*ptr).fy     = new float[(*ptr).nparticles];
  (*ptr).fy0    = new float[(*ptr).nparticles];
  (*ptr).fx0    = new float[(*ptr).nparticles];
  (*ptr).prev_x = new double[(*ptr).nparticles];
  (*ptr).prev_y = new double[(*ptr).nparticles];  
  (*ptr).neighbours = new int[19*(*ptr).nparticles];
}

/********************************************************************
 *  this function re-checks the posistions for overlaping, when any *
 *  of the particles overlap.                                       * 
 *******************************************************************/

void function_Xple_check(int pp,    // index of particle to check
			 Variables* ptr
			 )
{
   
  /* determine if the particles overlap -- and make new positions
   * if they do
   */
  
  for(int lp=0; lp<pp; lp++)
  {
    if( sqrt( SQUARE( (*ptr).x[pp] - (*ptr).x[lp] ) 
	    + SQUARE( (*ptr).y[pp] - (*ptr).y[lp] ) ) 
	      < 1 )
    {
      (*ptr).x[pp] =0.5 + ((*ptr).lx -1)  * lkiss();
      (*ptr).y[pp] =0.5 + ((*ptr).ly -1)  * lkiss();
      function_Xple_check(pp,ptr);
    }
  }
}


/********************************************************************
 *
 *  make initial positions and velocities
 *
 *********************************************************************
*/

void make_initial_state(Variables* ptr)
{
  int pp=1;	 // pp is  present center coordinate of particle
  float avg_velocity=0;

  (*ptr).x[0] =.5+ ((*ptr).lx-1)*lkiss();
  (*ptr).y[0] =.5+ ((*ptr).ly-1)*lkiss(); 
  
  for(pp=1;pp < (*ptr).nparticles;pp++)
  {	
    (*ptr).x[pp] =.5+ ((*ptr).lx-1)*lkiss();
    (*ptr).y[pp] =.5+ ((*ptr).ly-1)*lkiss();
    
    /* determine if the initial positions overlap and makes 
       new positions if they overlap                       */

    function_Xple_check(pp,ptr); 
  }
  
  /* give a random initial velocity */
  
  for(int j=0;j < (*ptr).nparticles;j++)
  {
    (*ptr).vx[j] = 2* (lkiss()-0.5);
    (*ptr).vy[j] = 2* (lkiss()-0.5);
    avg_velocity +=sqrt( SQUARE( (*ptr).vx[j] ) + 
			 SQUARE( (*ptr).vy[j] )  );
  }
  avg_velocity = avg_velocity/((*ptr).nparticles);
  MaxBoltzMemAlloc(avg_velocity,ptr);
} 



/*****************************************************************
 *                                                               *
 *  these are the energy routines, calculates the                *
 *  potential energy of each particle as well as the function    *
 *  for calculating the kinetic energy                           *
 *                                                               *
 *****************************************************************/



/* kinetic energy routine */

void kinetic_energy_calc(Variables* ptr)
{
  int i;
  (*ptr).kinetic_energy = 0;

  /* calculate kinetic energy */
  
  for( i=0; i < (*ptr).nparticles; i++ )
  { 
   (*ptr).kinetic_energy += ( (*ptr).vx[i] * (*ptr).vx[i] + (*ptr).vy[i] * (*ptr).vy[i] )/2;
  }
}



/* potential energy routine */

void potential_energy_calc(Variables* ptr)
{
  float	rsq, r6;		// dummies for r^2 and r^6
  float deltax, deltay;		// dummies for dx and dy
  float R6;                     // R_max^6
  float constant_energy;        // Value added on to the potential energy due to the R_max constraint
  int i=0, j=0, k=0;	       	// counters for loops

 
  (*ptr).potential_energy = 0.;
  
  
  /* calculate the same potential function, but replace r6 
     with R_max, then assign the value to R_max that way 
     we don't have to create another variable             */

  R6 = R_max*R_max*R_max*R_max*R_max*R_max;

  constant_energy = - ( (2*(*ptr).blennard) / R6 ) * ( -1+1/ (2*R6 ) );
 
  
  /* calculate potential energy */
  
  for( i=0; i<(*ptr).nparticles; i++ )
  {
                                                
    (*ptr).potential_energy +=(*ptr).g * (*ptr).y[i];   	// gravitational contribution  

    for( ; ; k++)
    {
      j = (*ptr).neighbours[k];

      if(j == -1)
	{
	  k++;
	  break;
	}
      /*
	this loop is used only to calculate the potential energies       
	between any ith and jth particle. Realize that x[i],y[i] are     
	the current particles whiles x[j], y[j] are the particle acting   
	on x[i],y[i]. This loop will calculate all the potential energies
	due to any ith particle caused by all nparticles and then add it 
	to the potential energy due to gravitation.                      
                                                                        */
      
      deltax = (*ptr).x[i] - (*ptr).x[j];
      deltay = (*ptr).y[i] - (*ptr).y[j];
      rsq = SQUARE(deltax) + SQUARE(deltay);
     
      /* at the same value, when R_max and r are the same, we can mathematically
	 prove that rsq = R_max^2, because R_max and r are equal. Anyway we will
	 compare the value of rsq and R_max to save computational time           */

      if(rsq >= R_maxSq)   
      {
	continue;
      }
      else
      {
	r6 = rsq*rsq*rsq;
    	(*ptr).potential_energy += ( (2*(*ptr).blennard) / r6 ) *
	( -1+1/ (2*r6 ) ) + constant_energy;
      }
    }

  }
}


/***************************************************************************
 *   this function does the force and neighbour list calculations.         *
 ***************************************************************************/

void Force_Calc(Variables* ptr)
{
  float	rsq=0, rsq3;	 	/* r square and rsq^3 */
  float deltax;                 /* holds the difference between any two x coordinates */
  float deltay;                 /* holds the difference between any two y coordinates */
  float f;			/* magnitude of force */
  float beff = -12*(*ptr).blennard;	/* effective lennard-jones parameter for forces calculateions */
  int i,j,k;                    /* counters */
  
  /* initialize the counters */

  i =0;
  j =0;
  k =0;

   /*  at time t = dt we would like to calculate all the accelerations     
       on an ith particle due to all other particles then .... Since     
       f=ma and m=1 in this particular case f=a, i.e. we will use fx[],   
       fy[] as our accelleration arrays                                   
                                                                           */                                   
    /* initialize the array to all zeros for fx, and -g for fy */
      
    for(i=0; i<(*ptr).nparticles; i++)
    {
      (*ptr).fx0[i] = (*ptr).fx[i];		// saves previous forces
      (*ptr).fy0[i] = (*ptr).fy[i];
      (*ptr).fx[i] = 0;		// initializes new forces 
      (*ptr).fy[i] = -(*ptr).g;
    }
  
    /* routine to update the neighbour list */ 
    
    for(i=0;i<(*ptr).nparticles;i++)
    {
      for(;;k++)
      {	
	j = (*ptr).neighbours[k];
	
	if( j == -1 )
	{
	  k++;
	  break;
	}

	deltax = (*ptr).x[i] - (*ptr).x[j];
	deltay = (*ptr).y[i] - (*ptr).y[j];
	
	rsq=( (deltax *  deltax ) + (deltay * deltay) );
      
	if(rsq >= R_maxSq)
	{
	  continue;
	}
	
	else
        {
	  rsq3=1./(rsq*rsq*rsq);
	  f=( beff*rsq3*(1.-rsq3) )/ rsq;
	  (*ptr).fx[i] += f*deltax;
	  (*ptr).fy[i] += f*deltay;
	  (*ptr).fx[j] += -f*deltax;
	  (*ptr).fy[j] += -f*deltay; 
	}

      }
    }
}   



/*********************************************************************
 *                                                                   *
 *  This function will set up and update the neighbour list.         *
 *                                                                   *
 *********************************************************************/

void Neighbour_List(Variables* ptr)
{
  float R_checkSq;              /* R_check squared  */
  float deltax;                 /* holds the difference between any two x coordinates */
  float deltay;                 /* holds the difference between any two y coordinates */
  float	rsq;     	 	/* r square  */
  int i,j,counter;              /* i,j are regular counters,and counter is for the 
				   neighbour list                                    */
  
  /************ effective parameters *************/
  
 
  R_checkSq = (R_check * R_check);
  counter   = 0;

  for(i=0;i<(*ptr).nparticles;i++)
  {
    for(j=i+1;j<(*ptr).nparticles;j++)
      {
	deltax = (*ptr).x[i]- (*ptr).x[j];
	deltay = (*ptr).y[i]- (*ptr).y[j]; 
	
	/* rsq^.5 = r; therefore R_max^2 = rsq */
	
	rsq=( (deltax *  deltax ) + (deltay * deltay) );
      
	if (rsq <= R_checkSq ) 
	{
	  (*ptr).neighbours[counter] = j;
	  counter++;
	}
      }
    (*ptr).neighbours[counter] = -1;  // add a negative 1 when the neighbour list ends
    counter++;
  }
  
  for(i=0;i<((*ptr).nparticles);i++)
  {
    (*ptr).prev_x[i] = (*ptr).x[i];            /* save the positions at the time of 
					   of updating the neighbour list */
    (*ptr).prev_y[i] = (*ptr).y[i];
  }
}


/************************the amazing shrinkin box routine***************/


/* really this stuff is amazing. */

static int check_x = FALSE;
static int check_y = FALSE;
void ShrinkTheBox(Variables* ptr)
{
     
  static float old_lx = (*ptr).lx;
  static float old_ly = (*ptr).ly;
  static float new_lx = (*ptr).lx;
  static float new_ly = (*ptr).ly;
  float t_lx = 0;          /* temporary lx, ly values */
  float t_ly = 0;
  char* string = new char[32];
 
  
  /* I need a way to check and see if the user wants to 
     keep shrinking the box, if this is the case then
     some shrinking proccess has to be enabled          */

  static int shrinking_ly_proccess = FALSE;
  static int shrinking_lx_proccess = FALSE;
  static int message = TRUE;


  /*******************shrinking box routine*************************/


  /* we would like to shrink the box slowly */


    /* shrink lx */

    if( old_lx > (*ptr).lx)
    {
      t_lx = (*ptr).lx;     /* shrunken value */
      (*ptr).lx = old_lx;   /* switch the big value with the small*/
      old_lx = t_lx;
      shrinking_lx_proccess = TRUE; 
    }
    
    if( new_lx < (*ptr).lx ) 
    {
      old_lx = (*ptr).lx;
      shrinking_lx_proccess = FALSE;
      check_x = FALSE;
    }

    /* lx wil keep shrinkig till it gets to the shrunken size we desire. 
       at that point, the difference between lx and the old value will
       be zero.                                                        */
 
    if((shrinking_lx_proccess == TRUE) && (check_x == TRUE )  )
      (*ptr).lx -= (((*ptr).lx - old_lx ) /5 ) * (*ptr).dt;  /* lx will never be negative */
     
    new_lx = (*ptr).lx;

    /* end of shrink lx */


  /* shrink ly */

    if( old_ly > (*ptr).ly )
    {
      t_ly = (*ptr).ly;     /* shrunken value */
      (*ptr).ly = old_ly;   /* switch the big value with the small*/
      old_ly = t_ly;
      shrinking_ly_proccess = TRUE;
    }
    
    if( new_ly < (*ptr).ly )
    {
      shrinking_ly_proccess = FALSE;
      old_ly = (*ptr).ly;
      check_y = FALSE;
    }


    /* ly wil keep shrinkig till it gets to the shrunken size we desire. 
       at that point, the difference between ly and the old value will
       be zero.                                                        */
 
    if( (shrinking_ly_proccess == TRUE) && ( check_y == TRUE))
      (*ptr).ly -= ( ( (*ptr).ly - old_ly ) /5 ) * (*ptr).dt;  /* ly will never be negative */
 
    new_ly = (*ptr).ly;

    /* end of shrink ly */


  /* 
     I. we the writers of this program, feel that it's exhustive work to have to
     subjuct molecules to such pressures. In our terms this is molecular abuse
     therefore we reserve the right not to subject these molecules to such
     crushing pressures. If ?% of the area in which these moleculdes lie
     is filled then, no more space will be sqeezed out of these precious molecules.
     Hence the else in this if statement.
     
     II. we also reserve the right to admit to any part taking in this molecular
     abuse. However if anything good comes of it, we would like to reserve the 
     credits for ourselves, yes that's us.

                           thank you very much.                       
     */

    if( ((*ptr).nparticles / ( (*ptr).lx * (*ptr).lx )
	>= .96 ) && ( message == TRUE ) )
    { 
      Widgets* wptr;
      wptr = ReturnClassAddress();
      message = FALSE;
      strcpy(string,"Danger! box too small ");
      MessageDialogCreationFunction((*wptr).dgr_message_window,string);
      shrinking_ly_proccess = FALSE;
      shrinking_lx_proccess = FALSE;
    }
    delete string;
  /****************************end of shrinking box routine*****************/
}

/********************************************************************
 * And now a moment of silence. Now hail to Wall_Pressure().        *
 * this is the ultimate function, it calculates the pressure on     *
 * the walls.                                                       *
 ********************************************************************/

void VWall_Pressure(Variables* ptr)    // y wall size
{
  int i=0;                     // counter 
  float instant_press_lx = 0;
  float instant_press_ly = 0;

  /* call the amazing shrink the box routine */
  
  ShrinkTheBox(ptr);


  // reverse normal components of velocity if colliding with wall
  // instant_pressure has the same value as fv_??, but it gets reset every graphics interval
  for(i=0;i<(*ptr).nparticles;i++)
  { 
    if ( ((*ptr).y[i] <= 0.5) && ((*ptr).vy[i] <= 0 ))  
    {
      (*ptr).vy[i] = -(*ptr).vy[i];
      (*ptr).fv_ly += (*ptr).vy[i] - ( - (*ptr).vy[i] ) ; // same as v_final - v_initial
      instant_press_ly += (*ptr).vy[i] - ( - (*ptr).vy[i] ) ;
    }

    if ( ( (*ptr).y[i] >= ((*ptr).ly - 0.5 )) && ((*ptr).vy[i] >= 0 ))
    {
      (*ptr).vy[i] = -(*ptr).vy[i];
      (*ptr).fv_ly += - ( (*ptr).vy[i] - (-(*ptr).vy[i]) ) ;
      instant_press_ly +=  - ( (*ptr).vy[i] - (-(*ptr).vy[i]) ) ;
    } 

    if( ( (*ptr).x[i] >= ((*ptr).lx - 0.5 )) && ( (*ptr).vx[i] >= 0 ) )
    {
      (*ptr).vx[i] = -(*ptr).vx[i];
      (*ptr).fv_lx += - ( (*ptr).vx[i] - ( - (*ptr).vx[i] ) ) ; 
      instant_press_lx +=  - ( (*ptr).vx[i] - ( - (*ptr).vx[i] ) ) ; 
    }
 
    if( ( (*ptr).x[i] <= 0.5) && ( (*ptr).vx[i] <= 0 ))
    {
      (*ptr).vx[i] = -(*ptr).vx[i];
      (*ptr).fv_lx += (*ptr).vx[i] - (-(*ptr).vx[i] );
      instant_press_lx += (*ptr).vx[i] - (-(*ptr).vx[i] );
    }
  }

  /* check and make sure there has been contact with the walls before
     calculating the instantaneous pressure else set it equal to zero */
  if( (instant_press_lx == !0) && (instant_press_ly != 0) )
  {
    (*ptr).instant_pressure = ( ( instant_press_lx + instant_press_ly ) / 
				( 2.0 * (*ptr).dt)) * ( 1/instant_press_lx + 1/instant_press_ly ) ; 
  }
  else
  {
    (*ptr).instant_pressure = 0;
  }
      
  if ( (*ptr).t - (*ptr).itime_checker >= (*ptr).ftime_interval )   
  {      
    (*ptr).itime_checker = (*ptr).t;
    (*ptr).total_pressure = ( ( (*ptr).fv_lx + (*ptr).fv_ly ) / 
	 (*ptr).ftime_interval * 2 ) * ( 1/(*ptr).lx + 1/(*ptr).ly ) ; 
    (*ptr).fv_ly = 0;
    (*ptr).fv_lx = 0;
  }

  if((*ptr).reset == TRUE)
  {
    (*ptr).itime_checker = 0; /* we need to update normally */
    (*ptr).reset = FALSE;
  }
}



/*********************************************************************
 * Calculate x y positions of particles using Newton's laws          *
 *	and returns the new time at the end of the run               *
 *********************************************************************/


float update_position(Variables* ptr)

{
  

  float const dt2 = (*ptr).dt/2;     /* convenient constant for calculating positions and volocity */  
  float delx,dely,RSq;        /* differences between x,y coordinates,RSq is r^2 */
  int i;                      /* counter */
  int internal=0;             /* temporary variable */
  float end_time=0;           /* this is the time interval that the forces,velocities etc get integrated.*/
  float speed=0;              /* external averge velocity */
  
  end_time = (*ptr).t+(*ptr).update_interval;

  /* This loop is used to calculate the x,y  positions of the molecules */ 

  while((*ptr).t<end_time) 
  {
    // update positions
    for( i=0; i<(*ptr).nparticles; i++)
    {
      (*ptr).x[i] = (*ptr).x[i] + ( (*ptr).vx[i] + (*ptr).fx[i] *dt2 ) * (*ptr).dt;
      (*ptr).y[i] = (*ptr).y[i] + ( (*ptr).vy[i] + (*ptr).fy[i] *dt2 ) * (*ptr).dt;
      if((*ptr).x[i] >= (*ptr).lx -.5 )
 	check_x = FALSE;
      if((*ptr).y[i] >= (*ptr).ly -.5 )
 	check_y = FALSE;
    }
 
    /* call to the force and neighbour calculation function */

     Force_Calc(ptr);

    // check to see wether the neighbour list needs updating
    
    for (i=0;i<(*ptr).nparticles;i++)
    {
      delx = ((*ptr).prev_x[i]) - ((*ptr).x[i]);
      dely = ((*ptr).prev_y[i]) - ((*ptr).y[i]);
      RSq = (delx * delx) + (dely * dely);      
      if( RSq >= ((R_check - R_max)/2) * ((R_check - R_max)/2) )
      {
	Neighbour_List(ptr);
	break;
      }
    }

    
    // update velocities with average forces 
    
    for(i=0;i<(*ptr).nparticles;i++)
    {
      /* find the correct bin position based on the velocity of the
	 particle, then increase the number of particles. 
	 the reason why these calculation come before the update
	 of the velocities it that, initially when the velocities are
	 given to the particles the average is calculated in order to 
	 create the histogram array. hence we need to update this info
	 before updating the velocities.                              */

      speed = sqrt(SQUARE((*ptr).vx[i]) + SQUARE((*ptr).vy[i]));
  
      internal = (int)floor(speed/(*ptr).binsize + .5); /* round up */
  
      if(internal <= bhv)      /* this should take care of array overflow */ 
	(*ptr).histogram[internal]++;
      (*ptr).vx[i] = (*ptr).vx[i] + ( ((*ptr).fx[i]) + (*ptr).fx0[i] ) * dt2;
      (*ptr).vy[i] = (*ptr).vy[i] + ( ((*ptr).fy[i]) + (*ptr).fy0[i] ) * dt2;
    }
   
    /* reverse normal components of velocity if colliding with wall */

    VWall_Pressure(ptr);
    
    /* update time */

    (*ptr).t = (*ptr).t + (*ptr).dt;
    check_x = TRUE;
    check_y = TRUE; 
  }
  
  return((*ptr).t);
  
}

XtWorkProcId id;              /* work_proc() id */


/*********************************************************************
 *       Work procedure function                                     *
 *                                                                   *
 *   work procedure is executed while program waits for events       *
 *     essentially calls update and draw circles                     *
 *                                                                   *
 *********************************************************************/
 
Boolean work_proc(XtPointer Xptr)
{
  Variables* ptr;

  ptr = (Variables*)Xptr;
  DrawCircles(ptr);       
  if((*ptr).t < (*ptr).end_time)
  {
    (*ptr).t = update_position(ptr);
    kinetic_energy_calc(ptr);
    potential_energy_calc(ptr);    
    (*ptr).total_energy = (*ptr).kinetic_energy + (*ptr).potential_energy;  	
  }
  return FALSE;
}
 

/*********************************************************************
 *
 *           command line parser. this routine checks all the 
 *          arguments passed to main then determines what
 *          appropriate actions need taken.                        
 *
 *********************************************************************
 */

Boolean CommandLineParser(int argc, char** argv,Variables* ptr,Boolean nographics)
{
  int particles = 0;

  char* arguments[]=
  {
    "-graph",
    "-nographics",
    "-lx",
    "-ly",
    "-nparticles",
    "-graphics_interval",
    "-pressure_interval",
    "-dt",
    "-end_time",
    "-seeds",
    "-blennard",
    "-sys_temp",
    "-gravity",
    "-dump_parameters",
  };

  /* There is always at least one command line argument, the name of 
     the program.
     */

  if( argc > 1 )
  {
    for(int i=1;i<argc;i++)
    {		
      if(strcmp(argv[i],arguments[0]) == 0)
      {
	(*ptr).mask = 11;	
      }
      if(strcmp(argv[i],arguments[1]) == 0)
      {
	nographics = TRUE;
      }
      if(strcmp(argv[i],arguments[2]) == 0)
      {
	++i;
	(*ptr).lx = (float)atof(argv[i]);
      }
      if(strcmp(argv[i],arguments[3]) == 0)
      {
	++i;
	(*ptr).ly = (float)atof(argv[i]); 
      }
      if(strcmp(argv[i],arguments[4]) == 0)
      {
	++i;
	particles = (int)atof(argv[i]);
      }
      if(strcmp(argv[i],arguments[5]) == 0)
      {
	++i;
	(*ptr).update_interval = (float)atof(argv[i]);
      }
      if(strcmp(argv[i],arguments[6]) == 0)
      {
	++i;
	(*ptr).ftime_interval = (float)atof(argv[i]);
      }
      if(strcmp(argv[i],arguments[7]) == 0)
      {
	++i;
	(*ptr).dt = (float)atof(argv[i+1]);
      }
      if(strcmp(argv[i],arguments[8]) == 0)
      {
	++i;
	(*ptr).end_time = (float)atof(argv[i]);
      }
      if(strcmp(argv[i],arguments[9]) == 0)
      {
	++i;
	(*ptr).seed1 = (unsigned long)atof(argv[i]);
	++i;
	(*ptr).seed2 = (unsigned long)atof(argv[i]);
	++i;
	(*ptr).seed3 = (unsigned long)atof(argv[i]);
      }
      if(strcmp(argv[i],arguments[10]) == 0)
      {
	++i;
	(*ptr).blennard = (float)atof(argv[i]);
      }
      if(strcmp(argv[i],arguments[11]) == 0)
      {
	++i;
	(*ptr).temp = (float)atof(argv[i]);
      }
      if(strcmp(argv[i],arguments[12]) == 0)
      {
	++i;
	(*ptr).g = (float)atof(argv[i]);
      }
      if(strcmp(argv[i],arguments[13]) == 0)
	 (*ptr).dump = 11;
    }
    if(particles > 0 )
      if(particles/(((*ptr).lx)*((*ptr).ly )) > 0.4)
      {
	(*ptr).lx = 6;
	(*ptr).ly = 6;
	(*ptr).nparticles = 10;
	if(nographics)
	 {
	   cout<<"the number of particles has been reset because of the 40% initial constraint"<<endl;
	   cout<<"nparticles = "<<(*ptr).nparticles<<endl;
	 }
      }
      else
	(*ptr).nparticles = particles; 
  }
  return nographics;

}/* end of the parser */


/* create a constructor for the class to initialize some variables */

Variables::Variables()
{
  nparticles =50;     /* number of particles */
  mask = 00;
  binsize=0;   
  blennard=1;         /* lennard jones constant */
  g=0;     	      /* gravity */
  lx=20 ;             /* x box size */
  ly=20;   	      /* y box size */
  dt=.001;            /* time step */
  t=0;		      /* time */
  end_time=0;	      /* ending time */
  update_interval=.02;/* used as interval between graphics calls */	
  seed1 =1;           /* seeds for random number generator */
  seed2 =2;
  seed3 =3;
  ndigit = 3;
  ftime_interval = 20;
  display_integral = 1;
  dump = 00;
}

/**********************************************************************
 * 
 *
 *	main routine --
 *
 ***********************************************************************
*/

/* the command line parameters are standards X switches */

int main (int argc, char** argv)
{  
  static Boolean nographics = FALSE;  /* for simulations without graphics */
  Variables *ptr = new Variables;     /* I made ptr so short because I have to use it in so many places */

  /* chech for any arguments first before setting up the system.*/
  
  nographics = CommandLineParser(argc,argv,ptr,nographics);

  /* setup the arrays */
  
  InitializeArrays(ptr);
  
  /* make the initial state and seeds */
       
  seedKiss((*ptr).seed1,(*ptr).seed2,(*ptr).seed3);   /* seeds random number gen.*/
  make_initial_state(ptr);                  /* makes initial positions and velocities*/

  /* create or update the neighbourlist */
  Neighbour_List(ptr);



  /***************** nongraphics update section *************/

  if(nographics)  
    NoGraphicsRoutine(ptr);  
 

/****************** graphics update *********************/

  if(!nographics)
  {
    /* graphics variables */
  
    XtAppContext dgr_app_context; /* the applications context for
				   *   for the event loop
				   */
    XEvent  event;        	/* the event variable structure contains
                         	 * information about the event that called
                         	 * the event handler
                         	 */
    
    dgr_app_context = SetupInitialGraphics(argc, argv,ptr);
    /* setup and initialize graphics and
     * draw initial state
     */
    
    /* call the work procedure function to update DrawCircles and some other functions */
    
    id = XtAppAddWorkProc(dgr_app_context, (XtWorkProc)work_proc,ptr);
    
    
    /* infinite loops while waiting for events */
    
    for(;;)
      {
	XtAppNextEvent(dgr_app_context, &event);	/* wait for event */
	XtDispatchEvent(&event);			/* handles the event */
	refreshDisplayAndScales(ptr);              /* rescale for resizes events */
      }
  }
  
}
                      






































